ANALYTICS_VALID_OPTION_KEYS = [:goal_id, 'goal_id', :start_date, 'start_date', :end_date, 'end_date', :taxon_id, 'taxon_id']


def get_registration_metric_row reg_id, metric, overrides={}
    spewy.post "/v0/registrations/#{reg_id}/metrics/#{metric}/rows", {
    }.merge(overrides)
end

def get_registration_expected_score reg_id, overrides={}
    get_registration_metric_row reg_id, 'expected-score', overrides
end

def get_registration_proficiency reg_id, overrides={}
    get_registration_metric_row reg_id, 'proficiency', overrides
end

def get_registration_active_time reg_id, overrides={}
    get_registration_metric_row reg_id, 'active-time', overrides
end

def get_registration_readiness_forecast reg_id, overrides={}
    get_registration_metric_row reg_id, 'readiness-forecast', overrides
end

def get_li_metric_row li_id, metric, overrides={}
    spewy.post "/v0/learning-instances/#{li_id}/metrics/#{metric}/rows", {
    }.merge(overrides)
end

def get_li_expected_score li_id, overrides={}
    get_li_metric_row li_id, 'expected-score', overrides
end

def get_li_proficiency li_id, overrides={}
    get_li_metric_row li_id, 'proficiency', overrides
end

def get_li_active_time li_id, overrides={}
    get_li_metric_row li_id, 'active-time', overrides
end

def get_li_readiness_forecast li_id, overrides={}
    get_li_metric_row li_id, 'readiness-forecast', overrides
end

def get_expected_score_for_reg_and_goal reg_id, goal_id
  spewy_get "/v0/registrations/#{reg_id}/metrics/expected-score?goal_id=#{goal_id}"
end


def get_expected_score_point_estimates_for_reg_and_goal reg_id, goal_id
  #return a trimmed down payload for easier inspection
  # HASH:  module_id -> point_estimate
  full_response = get_expected_score_for_reg_and_goal reg_id, goal_id
  trimmed = {}
  full_response['goals'][0]['target_modules'].each do |tm_data|
    trimmed[tm_data['module_id']] = tm_data['expected_scores'][0]['point_estimate']
  end
  trimmed
end


def get_expected_score reg_or_LI_label, reg_or_LI, options={}
  Log.info "Getting expected-score for #{reg_or_LI_label} #{reg_or_LI} with options: #{options}"
  #select valid options
  valid_options = options.values_at(*ANALYTICS_VALID_OPTION_KEYS)
  append_http_path = valid_options.zip(ANALYTICS_VALID_OPTION_KEYS).select{|v,k| v}.map{ |v,k| k.to_s + '=' + v}
  #select http path by label
  spewy.get "/v0/#{reg_or_LI_label.to_s+'s'}/#{reg_or_LI}/metrics/expected-score?" + append_http_path.join('&')
end

def get_active_time reg_or_LI_label, reg_or_LI, options={}
  Log.info "Getting active-time for #{reg_or_LI_label} #{reg_or_LI} with options: #{options}"
  #select valid options
  valid_options = options.values_at(*ANALYTICS_VALID_OPTION_KEYS)
  append_http_path = valid_options.zip(ANALYTICS_VALID_OPTION_KEYS).select{|v,k| v}.map{ |v,k| k.to_s + '=' + v}
  #select http path by label
  spewy.get "/v0/#{reg_or_LI_label.to_s+'s'}/#{reg_or_LI}/metrics/active-time?" + append_http_path.join('&')
end

def get_readiness_forecast reg_or_LI_label, reg_or_LI, options={}
  Log.info "Getting readiness-forecast for #{reg_or_LI_label} #{reg_or_LI} with options: #{options}"
  #select valid options
  valid_options = options.values_at(*ANALYTICS_VALID_OPTION_KEYS)
  append_http_path = valid_options.zip(ANALYTICS_VALID_OPTION_KEYS).select{|v,k| v}.map{ |v,k| k.to_s + '=' + v}
  #select http path by label
  spewy.get "/v0/#{reg_or_LI_label.to_s+'s'}/#{reg_or_LI}/metrics/readiness-forecast?" + append_http_path.join('&')
end

def get_proficiency reg_or_LI_label, reg_or_LI, options={}
  Log.info "Getting proficiency for #{reg_or_LI_label} #{reg_or_LI} with options: #{options}"
  #select valid options
  valid_options = options.values_at(*ANALYTICS_VALID_OPTION_KEYS)
  append_http_path = valid_options.zip(ANALYTICS_VALID_OPTION_KEYS).select{|v,k| v}.map{ |v,k| k.to_s + '=' + v}
  #select http path by label
  spewy.get "/v0/#{reg_or_LI_label.to_s+'s'}/#{reg_or_LI}/metrics/proficiency?" + append_http_path.join('&')
end

# spewy_get is handy for making adhoc calls right off the cmd line:
# kgrift -e QA -v spewy_get "/v0/someurl?someparam=abc"
def spewy_get full_url, options={}
  spewy.get full_url, options
end
