require 'rspec/expectations'

def eventually options={}, &blk
  timeout = options[:timeout] || 10
  interval = options[:interval] || 1
  time_limit = Time.now + timeout
  loop do
    begin
      yield blk
    rescue Grifter::RequestException => error
      raise error if error.code >= 500
    rescue RSpec::Expectations::ExpectationNotMetError, ExpectationError => error
      #nothing to check here... just keep waiting
    end
    return if error.nil?
    if Time.now >= time_limit
      Log.debug "eventually condition timed out:  #{error}"
      raise error
    end
    Log.debug "eventually condition not yet met, will keep trying for #{(time_limit - Time.now).to_i.to_s} seconds:  #{error}"
    sleep interval
  end
end


#this helper is useful for waiting until a POST'd object acutally creates
def wait_for_code expected_response_code=200, polling_options={}, &blk
  #these options, if set, will cause us to check the response that comes back.
  #to ensure we are getting the kind of asyc error we expect.
  #If another error happens, we will stop immediately
  error_code_must_be = polling_options[:error_code_must_be]
  error_message_must_match = polling_options[:error_message_must_match]

  eventually polling_options do
    begin
      yield blk
    rescue Grifter::RequestException => e
      #a 5xx should never happen, raise it if it does
      if e.code.to_i >= 500
        raise e
      end
      if error_code_must_be
        raise e unless e.code.to_i == error_code_must_be.to_i
      end
      if error_message_must_match
        raise e unless JSON.parse(e.body)['message'] =~ error_message_must_match
      end
    end
    raise e unless kapi.last_response.status.to_i == expected_response_code.to_i
  end
end

def time_it options={}, &blk
  start_time = Time.now
  yield blk
  end_time = Time.now
  block_took = (end_time - start_time)
  Log.info "TIMING LOG: It took #{block_took} to execute the block: #{options.fetch(:name, 'No block name given')}"
  return block_took
end

#Define new exception class for asynchronous expectation errors
class ::ExpectationError < RuntimeError; end
