
# return a list of all inventories
def get_inventories_v1
  Log.info "getting all inventories"
  coco.get "/v1/cc/admin/inventories"
end

def create_inventory_admin_v1 partner_id, new_inventory_id=nil
  new_inventory_id = "V1_KGriftTestInventory_#{random_string(16)}" if new_inventory_id.nil?
  Log.info "Admin creating an inventory - partner_id: #{partner_id} - inventory_id: #{new_inventory_id}"
  # response = create_modules_admin partner_id, new_inventory_id
  response = coco.post "/v1/cc/admin/#{partner_id}/#{URI.encode new_inventory_id}/inventory", ''

  {
    'id' => new_inventory_id,
    'inventory_id' => new_inventory_id,
    'partner_id' => partner_id,
    'inventory' => response,
  }
end

# this is used to make production inventories for real partners
def create_prod_inventory_admin_v1 partner_id, new_inventory_id
  set_prod_mode
  set_account :prod_knerd
  create_inventory_admin_v1 partner_id, new_inventory_id
end

def delete_inventory_admin_v1 partner_id, inventory_id
  Log.info "Admin deleting an inventory - partner_id: #{partner_id}"
  coco.delete "/v1/cc/admin/#{partner_id}/#{inventory_id}/inventory"
end

def get_all_v1_inventories_for_partner partner_id
  invs = get_inventories_v1['partner_inventories']
  invs.select!{|i| i['partner_id'] == partner_id}
end

def delete_all_v1_inventories_for_partner partner_id
  invs = get_inventories_v1['partner_inventories']
  invs.select!{|i| i['partner_id'] == partner_id}
  inv_ids = invs.map{|i| i['inventory_id']}
  inv_ids.each do |inv_id|
    delete_inventory_admin_v1 partner_id, inv_id
  end
end

def get_inventory_admin_v1 partner_id, inventory_id, query_params={}
  Log.info "Admin getting inventory - partner_id: #{partner_id} - inventory_id: #{inventory_id}"
  coco.get "/v1/cc/admin/#{partner_id}/#{inventory_id}?#{URI.encode_www_form query_params}"
end

def get_inventory_v1 inventory_id, query_params={}
  Log.info "getting inventory - inventory_id: #{inventory_id}"
  coco.get "/v1/cc/inventory/#{inventory_id}?#{URI.encode_www_form query_params}"
end

def check_inventory_existence_v1 partner_id, inventory_id
  Log.info "Admin checking inventory existence - partner_id: #{partner_id} - inventory_id: #{inventory_id}"
  coco.head "/v1/cc/admin/#{partner_id}/#{inventory_id}/inventory"
end

# build out a basic module object, ready to be created directly
# or you can merge in customizations
def test_module_object_v1 params={}
  params = {
    "module_id" => 'KGrift V1 test module ' + random_uuid,
    "module_name" => random_string,
    "module_version" => random_string,
    "module_type" => ['instruction', 'assessment', 'both'].sample,
    "module_url" => "http://kgriftlms.com/#{random_string}",
    'module_metadata' => ["some-test-metadata",random_string],
  }.merge(params)
end

# create a batch of modules against some inventory
def create_modules_v1 inventory_id, modules=[]
  modules << test_module_object_v1 if modules.empty?
  Log.info "Creating '#{modules.length}' modules for inventory_id: #{inventory_id}"
  coco.post "/v1/cc/inventory/#{inventory_id}/modules", modules
end
alias :update_modules_v1 :create_modules_v1

# create a batch of modules against some inventory
def create_modules_admin_v1 partner_id, inventory_id, modules=[]
  modules << test_module_object_v1 if modules.empty?
  Log.info "Admin creating '#{modules.length}' modules for inventory - partner_id: #{partner_id} - inventory_id: #{inventory_id}"
  coco.post "/v1/cc/admin/#{partner_id}/#{inventory_id}/modules", modules
end
alias :update_modules_admin_v1 :create_modules_admin_v1

# get a inventory module
def get_inventory_module_v1 inventory_id, module_id
  Log.info "Getting module - inventory_id: #{inventory_id} - module_id: #{module_id}"
  coco.get "/v1/cc/inventory/#{URI.encode inventory_id}/module/#{URI.encode module_id}"
end

# get a single inventory module
def get_inventory_module_admin_v1 partner_id, inventory_id, module_id
  Log.info "Admin getting module - partner_id: #{partner_id} - inventory_id: #{inventory_id} - module_id: #{module_id}"
  coco.get "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/module/#{URI.encode module_id}"
end

# delete a module
def delete_module_v1 inventory_id, module_id
  Log.info "Deleting inventory module - inventory_id: #{inventory_id} - module_id: #{module_id}"
  coco.delete "/v1/cc/inventory/#{inventory_id}/module/#{URI.encode module_id}"
end

# delete a module as a knerd
def delete_module_admin_v1 partner_id, inventory_id, module_id
  Log.info "Admin deleting inventory module - partner_id: #{partner_id} - inventory_id: #{inventory_id} - module_id: #{module_id}"
  coco.do_request :delete, "/v1/cc/admin/#{partner_id}/#{inventory_id}/module/#{URI.encode module_id}", {}
end

# mark a module mapped
def mark_module_mapped_v1 partner_id, inventory_id, module_id, module_version
  Log.info "Marking module mapped for inventory - partner_id: #{partner_id} - inventory_id: #{inventory_id} - module_id: #{module_id} - module_version: #{module_version}"
  coco.put "/v1/cc/admin/#{partner_id}/#{inventory_id}/map", {
    'module_id' => module_id,
    'module_version' => module_version,
  }
end

# this is a useful grift for testing purposes
# it will ensure an inventory has b/t 200 and 400 modules
# This is a good way to ensure that tests dont keep creating
# modules and never delete them
def attempt_to_set_number_of_test_modules partner_id, inventory_id, opts={}
  opts = {
    min: 150,
    max: 300,
  }.merge(opts)

  as_account :knerd do
    inv = get_inventory_admin_v1 partner_id, inventory_id
    num_mods = inv['modules'].length
    num_needed = opts[:min] - num_mods
    num_to_delete = num_mods - opts[:max]
    if num_needed > 0
      mods = num_needed.times.map{ test_module_object_v1}
      Log.info "Adding #{num_needed} modules to bring inventory to #{opts[:min]} modules"
      mods.each_slice(50) do |slice|
        create_modules_admin_v1 partner_id, inventory_id, slice
      end
    end
    if num_to_delete > 0
      Log.info "Deleting #{num_to_delete} modules to bring inventory to #{opts[:max]} modules"
      mod_ids = inv['modules'].map{|m| m['module_id']}.shuffle
      # lets be fast aboout it though and use concurrent requests...
      connection_pool = Typhoeus::Hydra.new(:max_concurrency => 8)
      responses = []
      coco.conn.in_parallel(connection_pool) do
        num_to_delete.times do |i|
          responses << kapi.conn.send(:delete) do |request|
            request.headers['Content-Type'] = 'application/json'
            request.headers['Authorization'] = kapi.headers['Authorization']
            request.url "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/module/#{URI.encode mod_ids[i]}"
          end
        end
      end
      response_codes = responses.map{|r| r.status }
      Log.debug "Responses:\n#{response_codes.join(', ')}"
    end
  end
  nil
end

def attempt_to_set_number_of_test_taxons partner_id, inventory_id, opts={}
  opts = {
    min: 150,
    max: 300,
  }.merge(opts)

  as_account :knerd do
    inv = get_inventory_taxons_admin_v1 partner_id, inventory_id
    num_taxons = inv['taxons'].length
    num_needed = opts[:min] - num_taxons
    num_to_delete = num_taxons - opts[:max]
    if num_needed > 0
      taxons = num_needed.times.map{ test_taxon_object_v1 }
      Log.info "Adding #{num_needed} taxons to bring inventory to #{opts[:min]} taxons"
      taxons.each_slice(50) do |slice|
        create_taxons_admin_v1 partner_id, inventory_id, slice
      end
    end
    if num_to_delete > 0
      Log.info "Deleting #{num_to_delete} taxons to bring inventory to #{opts[:max]} taxons"
      taxon_ids = inv['taxons'].map{|m| m['taxon_id']}.shuffle
      # lets be fast aboout it though and use concurrent requests...
      connection_pool = Typhoeus::Hydra.new(:max_concurrency => 8)
      responses = []
      coco.conn.in_parallel(connection_pool) do
        num_to_delete.times do |i|
          responses << kapi.conn.send(:delete) do |request|
            request.headers['Content-Type'] = 'application/json'
            request.headers['Authorization'] = kapi.headers['Authorization']
            request.url "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/taxon/#{URI.encode taxon_ids[i]}"
          end
        end
      end
      response_codes = responses.map{|r| r.status }
      Log.debug "Responses:\n#{response_codes.join(', ')}"
    end
  end
  nil
end

def attempt_to_set_number_of_test_containers partner_id, inventory_id, opts={}
  opts = {
    min: 150,
    max: 300,
  }.merge(opts)

  as_account :knerd do
    inv = get_inventory_containers_admin_v1 partner_id, inventory_id
    num_containers = inv['containers'].length
    num_needed = opts[:min] - num_containers
    num_to_delete = num_containers - opts[:max]
    if num_needed > 0
      containers = num_needed.times.map{ test_container_object_v1 }
      Log.info "Adding #{num_needed} containers to bring inventory to #{opts[:min]} containers"
      containers.each_slice(50) do |slice|
        create_containers_admin_v1 partner_id, inventory_id, slice
      end
    end
    if num_to_delete > 0
      Log.info "Deleting #{num_to_delete} containers to bring inventory to #{opts[:max]} containers"
      container_ids = inv['containers'].map{|m| m['container_id']}.shuffle
      # lets be fast aboout it though and use concurrent requests...
      connection_pool = Typhoeus::Hydra.new(:max_concurrency => 8)
      responses = []
      coco.conn.in_parallel(connection_pool) do
        num_to_delete.times do |i|
          responses << kapi.conn.send(:delete) do |request|
            request.headers['Content-Type'] = 'application/json'
            request.headers['Authorization'] = kapi.headers['Authorization']
            request.url "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/container/#{URI.encode container_ids[i]}"
          end
        end
      end
      response_codes = responses.map{|r| r.status }
      Log.debug "Responses:\n#{response_codes.join(', ')}"
    end
  end
  nil
end


def test_taxon_object_v1 params={}
  params = {
    "taxon_id" => 'TEST_TAXON:Foo|' + random_uuid,
    "taxon_name" => "TEST_TAXON_#{random_string}",
    "modules" => [],
    "containers" => []
  }.merge(params)
end

def create_taxons_admin_v1 partner_id, inventory_id, taxons=[]
  taxons << test_taxon_object_v1 if taxons.empty?
  Log.info "Admin creating taxons - partner_id: #{partner_id} - inventory_id: #{inventory_id} - taxons: #{taxons}"
  coco.post "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/taxons", taxons
end
alias :update_taxons_admin_v1 :create_taxons_admin_v1

def get_inventory_taxons_admin_v1  partner_id, inventory_id, query_params={}
  Log.info "Admin getting taxons - partner_id: #{partner_id} - inventory_id: #{inventory_id}"
  coco.get "/v1/cc/admin/#{partner_id}/#{inventory_id}/taxons?#{URI.encode_www_form query_params}"
end

def get_module_taxons_admin_v1  partner_id, inventory_id, module_id
  Log.info "Admin getting taxons - partner_id: #{partner_id} - inventory_id: #{inventory_id} - module_id: #{module_id}"
  coco.get "/v1/cc/admin/#{partner_id}/#{inventory_id}/taxons/#{URI.encode module_id}"
end

def delete_taxons_admin_v1 partner_id, inventory_id, taxon_id
  Log.info "Admin deleting taxons - partner_id: #{partner_id} - inventory_id: #{inventory_id} - taxon_id: #{taxon_id}"
  coco.delete "/v1/cc/admin/#{partner_id}/#{inventory_id}/taxon/#{URI.encode taxon_id}"
end

def create_taxons_v1 inventory_id, taxons=[]
  taxons << test_taxon_object_v1 if taxons.empty?
  Log.info "Partner creating taxons - inventory_id: #{inventory_id} - taxons: #{taxons}"
  coco.post "/v1/cc/inventory/#{URI.encode inventory_id}/taxons", taxons
end

def delete_taxons_v1 inventory_id, taxon_id
  Log.info "Partner deleting taxons - inventory_id: #{inventory_id} - taxon_id: #{taxon_id}"
  coco.delete "/v1/cc/inventory/#{inventory_id}/taxon/#{URI.encode taxon_id}"
end

def test_container_object_v1 params={}
  params = {
    "container_id" => 'TEST_CONTAINER_' + random_uuid,
    "modules" => [],
    "container_name" => nil,
    "container_url" => "http://kgriftlms.com/#{random_string(rand(10))}",
    "container_type" => ['presentation', 'target', 'both'].sample(),
    "container_metadata" => []
  }.merge(params)
end

def create_containers_admin_v1 partner_id, inventory_id, containers=[]
  containers << test_container_object_v1 if containers.empty?
  Log.info "Admin creating #{containers.length} containers - partner_id: #{partner_id} - inventory_id: #{inventory_id}"
  coco.post "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/containers", containers
end
alias :update_containers_admin_v1 :create_containers_admin_v1

def delete_containers_admin_v1 partner_id, inventory_id, container_id
  Log.info "Admin deleting containers - partner_id: #{partner_id} - inventory_id: #{inventory_id} - container_id: #{container_id}"
  coco.delete "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/container/#{URI.encode container_id}"
end

def get_containers_admin_v1 partner_id, inventory_id, module_id
  Log.info "Admin getting containers - partner_id: #{partner_id} - inventory_id: #{inventory_id} - module_id: #{module_id}"
  coco.get "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/containers/#{URI.encode module_id}"
end

def create_containers_v1 inventory_id, containers=[]
  containers << test_container_object_v1 if containers.empty?
  Log.info "Partner creating #{containers.length} containers - inventory_id: #{inventory_id}"
  coco.post "/v1/cc/inventory/#{URI.encode inventory_id}/containers", containers
end

def delete_containers_v1 inventory_id, container_id
  Log.info "Partner deleting containers - inventory_id: #{inventory_id} - container_id: #{container_id}"
  coco.delete "/v1/cc/inventory/#{URI.encode inventory_id}/container/#{URI.encode container_id}"
end

def get_inventory_containers_admin_v1  partner_id, inventory_id, query_params={}
  Log.info "Admin getting containers - partner_id: #{partner_id} - inventory_id: #{inventory_id}"
  coco.get "/v1/cc/admin/#{partner_id}/#{inventory_id}/containers?#{URI.encode_www_form query_params}"
end

def test_learning_objective_object_v1 params={}
  params = {
    'learning_objective_id' => "TEST_OBJECTIVE_#{random_uuid}",
    'learning_objective_name' => 'Test Learning Objective',
    'learning_objective_description' => "Able to test learning objectives on Knewton Platform",
    'modules' => [],
    'reference_modules' => [],
  }.merge(params)
end

def create_objectives_v1 inventory_id, objectives=[]
  objectives << test_learning_objective_object_v1 if objectives.empty?
  Log.info "Partner creating #{objectives.length} objectives - inventory_id: #{inventory_id}"
  coco.post "/v1/cc/inventory/#{URI.encode inventory_id}/objectives", objectives
end

alias :update_objectives_v1 :create_objectives_v1

def create_objectives_admin_v1 partner_id, inventory_id, objectives=[]
  objectives << test_learning_objective_object_v1 if objectives.empty?
  Log.info "Partner creating #{objectives.length} objectives - inventory_id: #{inventory_id}"
  coco.post "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/objectives", objectives
end

alias :update_objectives_admin_v1 :create_objectives_admin_v1

def get_objective_admin_v1 partner_id, inventory_id, objective_id
  Log.info "Admin getting objective - inventory_id: #{inventory_id} - partner_id: #{partner_id} - objective_id: #{objective_id}"
  coco.get "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/objective/#{objective_id}"
end

def get_objectives_admin_v1 partner_id, inventory_id
  Log.info "Admin getting objectives - inventory_id: #{inventory_id} - partner_id: #{partner_id}"
  coco.get "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/objectives"
end

def delete_objective_admin_v1 partner_id, inventory_id, objective_id
  Log.info "Admin deleting objective - inventory_id: #{inventory_id} - partner_id: #{partner_id} - objective_id: #{objective_id}"
  coco.delete "/v1/cc/admin/#{partner_id}/#{URI.encode inventory_id}/objective/#{objective_id}"
end


# get the transaction log of an inventory,
def get_inventory_log_v1 partner_id, inventory_id, params={}
  params = {
    'start' => days_from_now(-7),
    'coalesce' => false,
  }.merge(params)
  # format the timestamp if we need to
  if params['start'].respond_to? :utc
    params['start'] = params['start'].utc.iso8601(3)
  end
  Log.info "Getting inventory log - partner_id: #{partner_id} - inventory_id: #{inventory_id}"
  coco.post "/v1/cc/admin/#{partner_id}/#{inventory_id}/log", params
end

def ensure_inventory_does_not_exist_v1 partner_id, inventory_id
  delete_inventory_admin_v1 partner_id, inventory_id
rescue Grifter::RequestException => e
  # if the inventory does not an exist a 404 is returned.  For our purposes, that is fine
  # so we'll allow the request to fail if 404 comes back
  raise e unless e.code.to_i == 404
end

# make an inventory complete with
# - modules
# - containers
# - taxons
# - coming soon: Learning objectives
def create_v1_test_inventory partner_id, options={}
  options = {
    inventory_id: "Intro_to_#{random_string}",

    # concepts are not really made, but are used as an organizational element
    num_concepts: 2,
    # containers are made per concept
    num_assessing_containers_per_concept: 1,
    num_teaching_containers_per_concept: 2,

    # modules are made per container
    num_modules_per_assessing_container: 2,
    num_modules_per_teaching_container: 3,

    taxonomy_name: 'testtax',
    num_taxons: 5,
    num_modules_per_taxon: (3..20),
    num_containers_per_taxon: (0..2),

    # make this many modules|containers|taxons at a time
    batch_size: 50

  }.merge options

  options[:partner_id] = partner_id

  module_ids = nil
  container_ids = nil
  taxon_ids = nil

  as_account :knerd do

    inv_id = options[:inventory_id]
    create_inventory_admin_v1 partner_id, options[:inventory_id]

    containers = []
    modules = []

    options[:num_concepts].times do |concept_num|
      concept_id = "concept_#{concept_num}"

      # for each concept, setup the assessing and teaching containers
      options[:num_assessing_containers_per_concept].times do |container_num|
        container_id = "#{concept_id}_assess_cont_#{container_num}"

        container_mods = []
        options[:num_modules_per_assessing_container].times do |module_num|
          module_id = "#{container_id}_module_#{module_num}"
          container_mods << test_module_object_v1({ 'module_id' => module_id,
                                                    'module_type' => 'assessment' })
        end
        modules += container_mods

        containers << test_container_object_v1({'container_id' => container_id,
                                                'container_type' => 'target',
                                                'modules' => container_mods.map{|m| m['module_id']} })
      end

      # for each concept, setup the assessing and teaching containers
      options[:num_teaching_containers_per_concept].times do |container_num|
        container_id = "#{concept_id}_teach_cont_#{container_num}"

        container_mods = []
        options[:num_modules_per_teaching_container].times do |module_num|
          module_id = "#{container_id}_module_#{module_num}"
          container_mods << test_module_object_v1({ 'module_id' => module_id,
                                                    'module_type' => 'instruction' })
        end
        modules += container_mods

        containers << test_container_object_v1({'container_id' => container_id,
                                                'container_type' => 'presentation',
                                                'modules' => container_mods.map{|m| m['module_id']} })
      end

    end

    module_ids = modules.map{|m| m['module_id']}
    container_ids = containers.map{|c| c['container_id']}

    taxons = []
    options[:num_taxons].times do |taxon_num|
      taxon_id = "#{options[:taxonomy_name]}:#{taxon_num}_#{random_string(8)}"
      mods = module_ids.shuffle[0...rand(options[:num_modules_per_taxon])]
      taxons << test_taxon_object_v1({ 'taxon_id' => taxon_id,
                                       'modules' => mods })
    end

    taxon_ids = taxons.map{|t| t['taxon_id']}

    # now make all the objects in an order that will work:
    # - make all modules
    # - make all containers, which reference modules
    # - make all taxons, which reference modules and containers
    modules.each_slice(options[:batch_size]) do |batch|
      set_account :knerd  # this protects against expiring token
      create_modules_admin_v1 partner_id, inv_id, batch
    end

    containers.each_slice(options[:batch_size]) do |batch|
      set_account :knerd  # this protects against expiring token
      create_containers_admin_v1 partner_id, inv_id, batch
    end

    taxons.each_slice(options[:batch_size]) do |batch|
      set_account :knerd  # this protects against expiring token
      create_taxons_admin_v1 partner_id, inv_id, batch
    end

  end
  # returning the options is good b/c
  # it returns the partner_id and inventory_id in a Hash
  # with some other keys too
  options.merge 'module_ids' => module_ids,
    'container_ids' => container_ids,
    'taxon_ids' => taxon_ids
end
