GRIFTER_INSTANCES_BY_ENVIRONMENT = {}

def get_grifter_instance_for_env env
  if env
    unless GRIFTER_INSTANCES_BY_ENVIRONMENT[env]
      new_grifter_config = {environment: env}
      new_grifter_config[:config_file] = grifter_configuration[:config_file] if grifter_configuration[:config_file]
      GRIFTER_INSTANCES_BY_ENVIRONMENT[env] = Grifter.new new_grifter_config
    end
    GRIFTER_INSTANCES_BY_ENVIRONMENT[env]
  else
    self
  end
end


def copy_learning_instance options={}
  options = {
    'goal_target_score_override' => nil,
    'goal_days_from_now_override' => nil,
    'force_metrics' => false
  }.merge(options)

  source_environment = get_grifter_instance_for_env options['source_environment']

  original_learning_instance = nil
  original_partner_id = nil
  source_environment.instance_eval do
    set_account :knerd
    # we'll get the learning instance in this special way so we can also get the partner id of it
    # in case the graph_id is an eid, we'll need the partner id to translate it into an iid
    original_learning_instance = get_learning_instance options['learning_instance_id'], additional_headers: { 'X-Include-Partner-Id' => 'true' }
    original_partner_id = kapi.last_response['X-Partner-Id']
  end

  # figure out the graph id we want to use in our new learning instance
  graph_id = if options['graph_id']
    options['graph_id']
  elsif original_learning_instance['graph_id'] =~ /^gref-/
    source_environment.instance_eval do
      set_account :knerd
      translate_graph_gref_into_internal_id original_partner_id, original_learning_instance['graph_id']
    end
  else
    original_learning_instance['graph_id']
  end

  new_learning_instance = create_learning_instance 'name' => "test learning instance for interesting goals made #{Time.now}",
    'graph_id' => graph_id

  if options['goal_id']
    goal_id_list = [ options['goal_id'] ]
  elsif options['goal_id_list']
    goal_id_list = options['goal_id_list']
  end

  if goal_id_list
    new_goal_list = []
    for goal_id in goal_id_list
      goal_to_copy = source_environment.instance_eval do
                get_goal original_learning_instance['id'], goal_id
      end
      # the API will create a new id for this new goal
      goal_to_copy.delete('id')
      # if this is an MTMPTS/LO goal, delete the target_ids too
      for target_module in goal_to_copy['target_modules']
        if target_module.include?('target_id')
            target_module.delete('target_id')
        end
      end
      if options['goal_target_score_override']
        goal_to_copy['target_modules'].each {|tm| tm['target_score'] = options['goal_target_score_override'] }
      end
      if options['goal_days_from_now_override']
        goal_target_date_override = days_from_now options['goal_days_from_now_override']
        goal_to_copy['target_modules'].each {|tm| tm['target_date'] = goal_target_date_override }
      end
      if options['force_metrics']
        goal_to_copy['metrics_enabled'] = true
      end
      new_goal = create_goal new_learning_instance['id'], goal_to_copy
      new_goal_list << new_goal
    end
  else
    new_goal_list = [ nil ]  # previously-- if goal_id wasn't passed in, goal => nil was returned
  end
  if new_goal_list.length > 1
  {
    'learning_instance' => new_learning_instance,
    'goal_list' => new_goal_list,
  }
  else
  {
    'learning_instance' => new_learning_instance,
    'goal' => new_goal_list[0],
  }
  end
end


def prepare_graph_copy graph_id, options={}

  options = {
    remove_taxon_ids: true
  }

  graph = get_graph_and_cache_it graph_id

  Log.info "Copying graph: #{graph['id']}"

  orig_id = graph.delete 'id'
  graph['name'] = "Copy of #{orig_id}(#{graph['name']})"


  id_to_temp_id = {}
  graph['nodes'].each_with_index do |node, i|
    temp_id = get_mref_eid(i.to_s)
    node['temp_id'] = temp_id
    id_to_temp_id[node.delete('id')] = temp_id

    node['name'] = "#{node['subtype']}-#{random_string(8)}" unless node['name']

    if options[:remove_taxon_ids]
      node.delete 'taxon_ids'
    end

  end

  graph['edges'].each do |edge|
    edge['start_temp_id'] = id_to_temp_id[edge.delete('start')]
    edge['end_temp_id'] = id_to_temp_id[edge.delete('end')]
  end

  filename = "copy_#{orig_id}.json"
  Log.info "Writing #{filename}"
  File.open filename, 'w' do |f|
    f.write(JSON.pretty_generate(graph))
  end

end


def post_graph_from_file filename
  graph_obj = YAML.load_file filename

  p graph_obj['name']
  graph = create_graph graph_obj

  Log.info "Created graph: #{graph['id']}"
  graph['id']
end
