

# New get taxonomies endpoint
def get_taxonomies_for_graph (graph_id, overrides={})
  Log.info "Getting taxonomies for graph id: '#{graph_id}'."
  resp = nil
  do_as_knewton_partner overrides.delete('use_current_user') do
    resp = kapi.get "/v0/taxonomies/graph-id/#{graph_id}"
  end
  resp
end

# get get a taxonomy by graph id + taxonomy id
def get_taxonomy_for_graph (graph_id, taxonomy_id, overrides={})
  Log.info "Getting taxonomy '#{taxonomy_id}' for graph id: '#{graph_id}'."
  ts = get_taxonomies_for_graph graph_id
  ts.find{ |t| t['id'] == taxonomy_id}
end

def get_graph_summary id, overrides={}
  Log.info "getting graph by id: '#{id}'"
  kapi.get "/v0/graphs/#{id}/summary"
end

def get_all_graphs overrides={}
  Log.info "Getting all graphs with params '#{overrides.keys.join(', ')}'"
  kapi.get '/v0/graphs?' + URI.encode_www_form(overrides)
end

def get_all_graphs_for_partner partner_id, overrides={}
  overrides.merge!'partner_id' => partner_id
  get_all_graphs overrides
end

def get_internal_graph_id partner_id, graph_name
  graphs = get_all_graphs_for_partner partner_id
  graph = graphs.find{|g| g['name'] == graph_name}
  raise "Couldn't find graph for partner #{partner_id} with name #{graph_name}" unless graph
  graph['name']
end

def translate_graph_gref_into_internal_id partner_id, gref
  partner_graphs = get_all_graphs_for_partner partner_id
  wanted_name = gref.sub(/^gref-/,'')
  found = partner_graphs.find{|g| g['name'] == wanted_name}
  raise "No such graph with name '#{wanted_name}' for partner: #{partner_id}" unless found
  found['id']
end

def get_graph id, overrides={}
  Log.info "getting graph by id: '#{id}'"
  do_as_knewton_partner overrides.delete('use_current_user') do
    kapi.get "/v0/graphs/#{id}", timeout: 18000
  end
end

GRAPH_CACHE_DIR = File.join(Dir.home,'.kgrift/graph_cache')
def get_graph_and_cache_it id, overrides={}
  FileUtils::mkdir_p GRAPH_CACHE_DIR
  cache_file_name = File.join(GRAPH_CACHE_DIR, id + '.json')
  if File.exist?(cache_file_name)
    return JSON.parse(IO.read(cache_file_name))
  else
    graph = get_graph id, overrides
    File.open(cache_file_name, 'w'){|f| f.write JSON.pretty_generate(graph)}
    return graph
  end
end

def get_graphs_map_of_assessing_bundles_for_post_requisite_concepts_and_cache_it graph, overrides={}
  id = graph['id']
  FileUtils::mkdir_p GRAPH_CACHE_DIR
  cache_file_name = File.join(GRAPH_CACHE_DIR, id + '_postreqs_to_assessing_bndls.json')
  if File.exist?(cache_file_name)
    return JSON.parse(IO.read(cache_file_name))
  else
    assessing_bundles_for_postreq = get_map_of_assessing_bundles_for_post_requisite_concepts graph
    File.open(cache_file_name, 'w'){|f| f.write JSON.pretty_generate(assessing_bundles_for_postreq)}
    return assessing_bundles_for_postreq
  end
end

def graph_yaml id
  graph = get_graph_and_cache_it id
  File.open("#{id}.yml", 'w') {|f| f.write graph.to_yaml}
end

def get_graph_by_name name, overrides={}
  Log.info "getting graph by name: '#{name}'"
  do_as_knewton_partner overrides.delete('use_current_user') do
    kapi.get "/v0/graphs?name=#{URI.encode name}"
  end
end

def graph_exists? id, overrides={}
  Log.info "Checking if graph exists with ID: #{id}"
  do_as_knewton_partner overrides.delete('use_current_user') do
    kapi.head "/v0/graphs/#{id}"
  end
  return true
rescue RequestException => e
  return false if e.code == 404
  raise e
end

def do_as_knewton_partner use_current_user=false, &blk
  if use_current_user
    yield
  else
    as_account :knerd do
      yield
    end
  end
end


def get_node graph_id, node_id, overrides={}
  Log.info "Getting a node from graph: #{graph_id} -> #{node_id}"
  kapi.get "/v0/graphs/#{graph_id}/nodes/#{node_id}"
end

def get_learning_objective_node graph_id, learning_objective_id, overrides={}
  Log.info "Getting a learning objective node from graph: #{graph_id} -> #{learning_objective_id}"
  kapi.get "/v0/graphs/#{graph_id}/learningObjectives/#{learning_objective_id}"
end

def get_edge graph_id, start_node_id, end_node_id, edge_type, overrides={}
  Log.info "Getting an edge from graph: #{graph_id} -> #{start_node_id}_#{edge_type}_#{end_node_id}"
  kapi.get "/v0/graphs/#{graph_id}/edges/#{start_node_id}_#{edge_type}_#{end_node_id}"
end

def get_edges graph_id, node_id, overrides={}
  Log.info "Getting a edges from graph #{graph_id}, starting from #{node_id}"
  kapi.get "/v0/graphs/#{graph_id}/nodes/#{node_id}/edges"
end

