"""
GraphTools is the rest api begind the Bramble react application

These grifts are used to power tests made directly against the graphtools
api.
"""

# the way we authenticate for graphtools api is different from other services
# other services use the knewton oauth2 provider.
# graphtools uses google oauth2 provider in such a way that restricts the app/api to only those with @knewton.com emails
#
# The complexity of getting a google token is completely hidden in the graphtools_api_token test data generator
def graphtools_authenticate
  # these google tokens last something like 12ish hours...
  # so we use the session_only option to get a single fresh one on each test run
  token = get_test_data(:graphtools_api_token, session_only: true)['token']
  graphtools.headers['Authorization'] = token
  # return true to mean success
  true
end

# Methods that adds additional headers needed by the graph editor endpoints
def graph_editor_get partner_id, inventory_id, url
  graphtools.get url, additional_headers: {
    'Partner-Id' => partner_id,
    'Inventory-Id' => inventory_id,
  }
end

def graph_editor_post partner_id, inventory_id, url, body
  graphtools.post url, body, additional_headers: {
    'Partner-Id' => partner_id,
    'Inventory-Id' => inventory_id,
  }
end

def graph_editor_delete partner_id, inventory_id, url, body = {}
  graphtools.do_request :delete, url, body, additional_headers: {
    'Partner-Id' => partner_id,
    'Inventory-Id' => inventory_id,
  }
end

# Endpoints for graph editor
# Pulling all modules from CoCo for a specific inventory
def pull_modules_from_coco partner_id, inventory_id
  graph_editor_post partner_id, inventory_id, "/graph", {}
end

# Clearing all the concepts and modules for a specific inventory
def clear_graph_tools partner_id, inventory_id
  graph_editor_post partner_id, inventory_id, "/graph/clear", {}
end

# Getting the difference between modules in CoCo and GraphTools
def graph_diff partner_id, inventory_id
  graph_editor_get partner_id, inventory_id, "/graph/delta"
end

# Getting number of modules that are different in CoCo and GraphTools
def graph_diff_summary partner_id, inventory_id
  graph_editor_get partner_id, inventory_id, "/graph/delta-summary"
end

# Create concept for an inventory
def graphtools_create_concept partner_id, inventory_id, concept={}
  graph_editor_post partner_id, inventory_id, "/concepts/concept", concept
end

# Update concept for an inventory
def graphtools_update_concept partner_id, inventory_id, concept={}
  graph_editor_post partner_id, inventory_id, "/concepts/concept/update", concept
end

# Get a specific concept for an inventory
def graphtools_get_concept partner_id, inventory_id, concept_id
  graph_editor_get partner_id, inventory_id, "/concepts/concept?#{URI.encode_www_form concept_id}"
end

# Delete a specific concept for an inventory
def graphtools_delete_concept partner_id, inventory_id, concept_id
  graph_editor_delete partner_id, inventory_id, "/concepts/concept/#{URI.encode concept_id}"
end

# Get a specific module from an inventory
def graphtools_get_module partner_id, inventory_id, module_id
  graph_editor_get partner_id, inventory_id, "/modules/module?#{URI.encode_www_form module_id}"
end

# Getting modules from an inventory
def graphtools_get_modules partner_id, inventory_id, query_params={}
  graph_editor_get partner_id, inventory_id, "/modules?#{URI.encode_www_form query_params}"
end
# Getting modules from an inventory
def graphtools_get_concepts partner_id, inventory_id, query_params={}
  graph_editor_get partner_id, inventory_id, "/concepts?#{URI.encode_www_form query_params}"
end

# Create a concept-concept edge and return the updated concept
def graphtools_create_cc_edge partner_id, inventory_id, edge
  graph_editor_post partner_id, inventory_id, "/concepts/edge/concept", edge
end
# Delete a concept-concept edge and return the updated concept
def graphtools_delete_cc_edge partner_id, inventory_id, edge
  graph_editor_delete partner_id, inventory_id, "/concepts/edge/concept", edge
end

# Create a concept-module edge and return the updated concept
def graphtools_create_cm_edge partner_id, inventory_id, edge
  graph_editor_post partner_id, inventory_id, "/concepts/edge/module", edge
end
# Delete a concept-module edge and return the updated concept
def graphtools_delete_cm_edge partner_id, inventory_id, edge
  graph_editor_delete partner_id, inventory_id, "/concepts/edge/module", edge
end

# Create a concept-module edge and return the updated module
def graphtools_create_mc_edge partner_id, inventory_id, edge
  graph_editor_post partner_id, inventory_id, "/modules/edge/concept", edge
end
# Delete a concept-module edge and return the updated module
def graphtools_delete_mc_edge partner_id, inventory_id, edge
  graph_editor_delete partner_id, inventory_id, "/modules/edge/concept", edge
end

# this is a unique function in that it writes a spreadsheet file
# the return is the path to the spreadsheet
def graphtools_get_graph_spreadsheet partner_id, inventory_id
  # we need to customize the headers for this to work...
  # so this request is being constructed in a very custom way...
  url = "/graph/spreadsheet"
  response = graphtools.get url, additional_headers: {
    'Partner-Id' => partner_id,
    'Inventory-Id' => inventory_id,
    'accept' => '*/*',
  }
  # response is the binary of the spreadsheet.  Just dump it into a file...
  # fist make a path to a file
  # path is designed to be a) findable by including inventory_id / partner_id
  # b) random, so that no possibility of tests using wrong file occurs
  tmp_folder = 'tmp'
  suppress(Exception) { Dir.mkdir tmp_folder }
  filename = "test_spreadsheet_#{partner_id}_#{inventory_id}_#{random_string(8)}.xlsx"
  path = File.join tmp_folder, filename
  # second write the response into the file
  File.open(path, 'w') do |f|
    f.write response
  end
  path
end

