require 'securerandom'
require 'time'
#magic date math methods come from active support
require 'active_support/core_ext/date/calculations'
require 'active_support/core_ext/numeric/time'
require 'digest'

module ::TestDataHelpers
def random_uuid
  SecureRandom.uuid
end

def uuid_to_bit_string uuid
  #remove dashes
  #convert into hex, and back into a base 2 number/string
  #pad the left with 0s
  uuid.gsub('-','').to_i(16).to_s(2).rjust(128, '0')
end

def is_test_uuid? uuid
  #the 41st bit is the test bit
  uuid_to_bit_string(uuid)[40] == '1'
end

def random_string length=32
  # to get the right length, for both both even and odd lengths,
  # we have to pull a couple little tricks given that
  # SecureRandom.hex returns twice the length requested
  # SecureRandom.hex though is the best way I've found to get
  # good random printable strings with a basic character set
  s = SecureRandom.hex((length/2) + 1).to_s # add one so get a bit more than we need
  s = s.force_encoding 'utf-8'
  s[0...length]  #chop the string to the length needed
end

# uses chinese characters
ChineseChars = [*"\u4E00".."\u9FFF"]
def random_chinese_string length=32
  length.times.map{ ChineseChars.sample }.join('')
end

def timestamp_string
  Time.now.strftime('%m%d%y%H%M%S')
end

def random_graph_name
  "TestGraph_#{random_string}"
end

def random_taxonomy_name
  "TestTaxonomy_#{random_string}"
end

def random_learning_instance_name
  "TestLearningInstance_#{random_string}"
end

def datetime_now
  #rfc-3339 format 2013-03-13T16:06:06-04:00
  #which is same as iso8601 (except in the very limited case of 00 timezones)
  #http://en.wikipedia.org/wiki/ISO_8601#Usage
  Time.now.utc.round.iso8601(3)
end
alias :datetime :datetime_now

def days_from_now num_days
  #for cleanliness days timestamps are all at midnight
  num_days.to_i.days.from_now.utc.beginning_of_day.round.iso8601(3)
end

def bad_json_payloads
  ["", "boomdeyada"]
end

def get_file_sha1 path
  Digest::SHA1.hexdigest(File.read(path))
end

def get_mref_eid label
  "mref-#{label}-#{random_string(20)}"
end

def get_gref_eid label
  "gref-#{label}-#{random_string(20)}"
end

def get_tref_eid label
  "tref-#{label}-#{random_string(20)}"
end

end
include ::TestDataHelpers
