## Grifts to make requests against the Moxy endpoints for partner load testing
##    with randomized responses.

def moxy_send_ungraded_event registration_id, module_id, overrides={}
  Log.info "Sending a moxy ungraded event"
  kapi.post "/v0/moxy/registrations/#{registration_id}/ungraded-events", {
    'duration' =>             876543,
    'module_id' =>            module_id,
    'is_complete' =>          true,
    'interaction_end_time' => datetime,
  }.merge(overrides)
end

def moxy_send_graded_event registration_id, module_id, overrides={}
  Log.info "Sending a moxy graded event"
  kapi.post "/v0/moxy/registrations/#{registration_id}/graded-events", {
    'response' =>             "sample of a response",
    'duration' =>             4321,
    'module_id' =>              module_id,
    'interaction_end_time' => datetime,
    'score' =>                0.99,
    'is_correct' =>           true,
    'is_complete' =>          true,
  }.merge(overrides)
end

def moxy_send_batch_events registration_id, events, overrides={}
  Log.info "Sending a moxy batch of focus events"
  kapi.post "/v0/moxy/registrations/#{registration_id}/batch-events", {
    'goal_id' => nil,
    'events' => events
  }.merge(overrides)
end

def moxy_send_focus_event registration_id, goal_id, overrides={}
  Log.info "Sending a moxy focus event"
  kapi.post "/v0/moxy/registrations/#{registration_id}/focus-events", {
    'goal_id' => goal_id,
  }.merge(overrides)
end

def moxy_get_recommendation registration_id, goal_id=nil, continued_recommendations=nil
  goal_id = random_uuid unless goal_id  #this only works in local mode, considering removing it
  Log.info "Getting a moxy recommendation for registration: " + registration_id + " goal: " + goal_id
  query_params= { 'goal_id' => goal_id }
  query_params['continued_recommendations'] = continued_recommendations if continued_recommendations
  query_string = URI.encode_www_form(query_params)
  kapi.get "/v0/moxy/registrations/#{registration_id}/recommendation?#{query_string}"
end

def moxy_get_registration_metrics registration_id, metric_name, options
  Log.info "Getting moxy analytics for registration: " + registration_id + " metric: " + metric_name
  kapi.get "/v0/moxy/registrations/#{registration_id}/metrics/#{metric_name}" + '?' + options.join('&')
end

def moxy_get_learning_instance_metrics learning_instance_id, metric_name, request_body={}
  Log.info "Getting moxy analytics for learning instance: " + learning_instance_id + " metric: " + metric_name
  kapi.post "/v0/moxy/learning-instances/#{learning_instance_id}/metrics/#{metric_name}/rows", request_body
end

def moxy_get_learning_instance_group_metrics learning_instance_id, request_body={}
  Log.info "Getting moxy groups analytics for learning instance: " + learning_instance_id
  kapi.post "/v0/moxy/learning-instances/#{learning_instance_id}/metrics/groups", request_body
end
