def knerd_token
  #this is a handy helper for other apps to quickly get a bearer token
  #run it like:  grift -e qa -q -n knerd_token
  set_account :knerd
  kapi.headers['Authorization']
end

def authenticate profile
  #if !profile.has_key?('username')
  #  profile.merge!('username'=>random_string)
  #end
  #if !profile.has_key?('password')
  #  profile.merge!('password'=>random_string)
  #end

  code = get_authorization_grant(profile)['code']
  token = get_access_token(
    'code' => code,
    'client_credentials' => profile.select{|k| k=='client_id' or k=='client_secret'},
    'external_user_id' => profile['external_user_id'],
  )['access_token']
  set_authorization_headers token
end

def get_authorization_grant parameters={}
  Log.info "Authenticating as user '#{parameters['username']}'"
  parameters = {
      'response_type' => "code",
      'client_id' => nil,
      'redirect_uri' => "http://client-redirect-uri",
  }.merge(parameters).delete_if{|k,v| v.nil?}
  kapi.post_form "/v0/oauth/authorize", parameters
end

def get_access_token options
  raise ArgumentError.new("options must be a Hash") unless options.is_a? Hash
  auth_header = construct_basic_auth_header_overrides options.delete 'client_credentials'
  Log.info "Getting an access token"
  params = {
    'code' => options['code'],
    'grant_type' => "authorization_code",
    'redirect_uri' => "http://client-redirect-uri",
    'external_user_id' => random_string,
  }.merge(options).delete_if{|k,v| v.nil?}
  kapi.post_form '/v0/oauth/token', params, additional_headers: auth_header
end

def get_access_token_from_refresh_token options
  raise ArgumentError.new("options must be a Hash") unless options.is_a? Hash
  auth_header = construct_basic_auth_header_overrides options.delete 'client_credentials'
  Log.info "Getting an access token from a refresh token"
  params = {
    'refresh_token' => options['refresh_token'],
    'grant_type' => "refresh_token",
    'redirect_uri' => "http://client-redirect-uri",
    'external_user_id' => random_string,
  }.merge(options)
  kapi.post_form '/v0/oauth/token', params, additional_headers: auth_header
end

def unauthenticate
  set_authorization_headers nil
end

def get_current_account
  kapi.get '/v0/accounts/current'
end

def get_account_by_id account_id
  kapi.get "/v0/accounts/#{account_id}", additional_headers: { 'X-Include-Partner-Id' => 'true' }
end

def get_account_by_username partner_id, username
  kapi.get "/v0/accounts/?partner_id=#{partner_id}&username=#{username}"
end

def get_account_id_for external_user_id, client_credentials=nil
  as_account external_user_id, client_credentials do
    get_current_account['id']
  end
end

def update_account account_id, overrides={}
  kapi.put "/v0/accounts/#{account_id}", {}.merge(overrides)
end

def create_account overrides={}
  kapi.post "/v0/accounts", {
    'external_user_id' => random_string,
    'entitlements' => []
  }.merge(overrides)
end

def delete_account account_id
  response = kapi.delete "/v0/accounts/#{account_id}"
  # a deleted account shouldn't have access to the api
  # so any tokens we have cached are no good
  # throwing them away is important to ensure certain integration tests work correctly
  delete_account_id_from_token_cache account_id
  response
end

def create_account_with_username_password_for_partner username, password, partner_id, overrides={}
  kapi.post "/v0/looper/_/accounts/", {
    'partner_id' => partner_id,
    'username' => username,
    'password' => password,
    'external_user_id' => random_string,
    'entitlements' => []
  }.merge(overrides)
end

def make_account_partner_admin account_id
  update_account account_id, 'entitlements' => ['all']
end

def update_looper_account_entitlements_by_username looper_partner_id, username, entitled_partner_ids
  as_account :knerd do
    account_response = get_account_by_username looper_partner_id, username
    update_looper_account_entitlements account_response['id'], entitled_partner_ids
  end
end

def update_looper_account_entitlements account_id, entitled_partner_ids
  kapi.put "/v0/accounts/#{account_id}", {
    'id' => account_id,
    'entitlements' => ['partner_admin'],
    'entitlements_metadata' => { 'partner_admin' => entitled_partner_ids.map { |id| { 'partner_id' => id } } }
  }
end


# This is a list of services that is not authenticated using knewton oauth
# graphtools is authenticated using google oauth2
# see graphtools_authenticate in graphtools_grifts
NON_AUTH_SERVICES=[:graphtools]

def set_authorization_headers token
  if token
    formatted_token = case token.to_sym
                    when /^\S+$/  #no spaces
                      'Bearer ' + token
                    else
                      token
                    end
  end
  svcs = grifter_configuration[:services].keys
  svcs.each do |svc|
    #  do not set the header for the special services authenticated in other ways
    next if NON_AUTH_SERVICES.include? svc
    if formatted_token
      self.send(svc).headers['Authorization'] = formatted_token
    else
      self.send(svc).headers.delete 'Authorization'
    end
  end
  formatted_token  #return the actual token, I guess
end

#this helper is used for adding the basic http auth parameter where needed in a standardized way
#it can be given either client_credentials or client_id/client_secret separately
require 'base64'
def construct_basic_auth_header_overrides options={}
  options={} unless options!=nil
  #client credentials given
  client_id=nil
  client_secret=nil
  if options.has_key? 'client_credentials'
    client_id = options['client_credentials']['client_id']
    client_secret = options['client_credentials']['client_secret']
  #client id and secret given separately
  elsif options.has_key? 'client_id' and options.has_key? 'client_secret'
    client_id = options['client_id']
    client_secret = options['client_secret']
  #no sufficient information (use default partner)
  else
    client_id = get_client_id
    client_secret = get_client_secret
  end
  encoded_token = Base64.strict_encode64(client_id + ':' + client_secret)
  {'Authorization' => 'Basic ' + encoded_token}
end
