##STUFF RELATING TO REGISTRATION RESOURCES
def create_registration overrides={}
  Log.info "Creating a registration"
  # if account_id was passed in, we'll use that account id
  # but, if account_id not provided, we'll make an account and return its details
  account = nil
  unless overrides['account_id']
    account = create_account
    overrides['account_id'] = account['id']
  end
  reg = kapi.post '/v0/registrations', {
     'type' => 'learner',
     'account_id' => nil,
     'learning_instance_id' => '456',
  }.merge(overrides)
  reg['external_user_id'] = account['external_user_id'] if account
  reg
end

def make_account_instructor_of account_id, learning_instance_id
  Log.info "Creating a registration of type instructor for learning instance " + learning_instance_id + " account_id " + account_id
  create_registration 'learning_instance_id' => learning_instance_id,
                      'account_id' => account_id,
                      'type' => "instructor"
end


def add_new_student_to_learning_instance learning_instance_id, client_credentials=nil, options={}
  client_credentials = get_client_credentials(:knewton) unless client_credentials
  account = create_account
  external_user_id = account['external_user_id']
  account_id = account['id']
  registration = create_registration 'learning_instance_id' => learning_instance_id,
                                     'account_id' => account_id,
                                     'type' => options.fetch('type', 'learner')
  {
    'external_user_id' => external_user_id,
    'registration_id' => registration['id'],
    'account_id' => account_id,
  }
end

def get_registration id
  Log.info "Getting a registration " + id
  kapi.get "/v0/registrations/#{id}"
end

def delete_registration id
  Log.info "Deleting a registration " + id
  kapi.delete "/v0/registrations/#{id}"
end

def get_registrations_for_account account_id
  Log.info "Getting a registration for account " + account_id
  kapi.get "/v0/accounts/#{account_id}/registrations"
end


##GOAL ACTIVATATION STUFF
def activate_registration_on_goal learning_instance_id, registration_id, goal_id
  Log.info "Activating a goal for a registration"
  kapi.put "/v0/learning-instances/#{learning_instance_id}/goals/#{goal_id}/registrations/#{registration_id}", {}
end

def deactivate_registration_on_goal learning_instance_id, registration_id, goal_id
  Log.info "Deactivating a goal for a registration"
  kapi.delete "/v0/learning-instances/#{learning_instance_id}/goals/#{goal_id}/registrations/#{registration_id}"
end

def list_activated_registrations learning_instance_id, goal_id
  Log.info "Listing activated registrations for a goal"
end

def check_if_activated learning_instance_id, registration_id, goal_id
  Log.info "Checking if a specific registration is activated for a specific goal"
  kapi.get "/v0/learning-instances/#{learning_instance_id}/goals/#{goal_id}/registrations/#{registration_id}"
  return true
rescue Grifter::RequestException => e
  if e.code == 404
    return false
  else
    raise e
  end
end


##SENDING IN EVENTS STUFF
def send_recommendation_followed registration_id, module_id, overrides={}
  Log.info "Sending a recommendation followed event"
  kapi.post "/v0/registrations/#{registration_id}/recommendation-followed-events", {
    'module_id' => module_id,
    'time_followed' => datetime,
    'recommendation_id' => 987654,
  }.merge(overrides)
end

def send_graded_event registration_id, module_id, overrides={}
  Log.info "Sending a graded event"
  kapi.post "/v0/registrations/#{registration_id}/graded-events", {
    'response' =>             "sample of a response",
    'duration' =>             4321,
    'module_id' =>              module_id,
    'interaction_end_time' => datetime,
    'score' =>                0.99,
    'is_correct' =>           true,
    'is_complete' =>          true,
  }.merge(overrides)
end

def send_ungraded_event registration_id, module_id, overrides={}
  Log.info "Sending an ungraded event"
  kapi.post "/v0/registrations/#{registration_id}/ungraded-events", {
    'duration' =>             876543,
    'module_id' =>            module_id,
    'is_complete' =>          true,
    'interaction_end_time' => datetime,
  }.merge(overrides)
end

def send_focus_event registration_id, goal_id, overrides={}
  Log.info "Sending a focus event"
  kapi.post "/v0/registrations/#{registration_id}/focus-events", {
    'goal_id' => goal_id,
  }.merge(overrides)
end

def send_random_event_on_graph registration_id, graph
    atom_id = find_random_atom_id graph
    send_correct_event_type_for_atom registration_id, atom_id, graph
end

def send_correct_event_type_for_atom registration_id, atom_id, graph
    content_types = get_content_types graph, atom_id

    if content_types.include? 'assesssment'
        correct = [true, false].sample
        send_graded_event registration_id, atom_id, is_correct: correct
    else
        send_ungraded_event registration_id, atom_id
    end
end

## GETTING RECOMMENDATION STUFF
def get_recommendation registration_id, goal_id=nil, continued_recommendations=nil
  goal_id = random_uuid unless goal_id  #this only works in local mode, considering removing it
  Log.info "Getting a recommendation for registration: " + registration_id + " goal: " + goal_id
  query_params= { 'goal_id' => goal_id }
  query_params['continued_recommendations'] = continued_recommendations if continued_recommendations
  query_string = URI.encode_www_form(query_params)
  kapi.get "/v0/registrations/#{registration_id}/recommendation?#{query_string}"
end

def send_batch_events registration_id, events, overrides={}
  Log.info "Sending a batch of student events"
  kapi.post "/v0/registrations/#{registration_id}/batch-events", {
    'goal_id' => nil,
    'events' => events
  }.merge(overrides)
end
